import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;

/**
 * Representa una ficha del juego Mahjong
 */
public class Ficha extends Actor
{
    /** El valor de la ficha, algo como "bamboo2" o "man9" */
    private String valor;
    
    /**
     * El nivel de profundidad de esta ficha respecto a otras, entre mas grande mas cerca del jugador.
     * Este numero ayuda a saber si una ficha esta encima de otra o esta al mismo nivel que otra.
     */
    private int nivelZ = 0;
    
    /** La imagen original de la ficha */
    private GreenfootImage image;
    
    /** Color usado para indicar que la ficha está seleccionada **/
    private static Color colorSeleccion = new Color(.5f, .8f, .8f, .33f);
    
    /**
     * Construye una ficha de Mahjong con el valor dado
     * @param valor Un texto como "bamboo2" o "man9"
     */
    public Ficha(String valor)
    {
        this.valor = valor;
        setImage(image = new GreenfootImage(valor + ".png"));
    }
    
    /** Retorna el valor de la ficha, algo como "bamboo2" o "man9" */
    public String valor()
    {
        return valor;
    }
    
    /**
     * Indica el nivel de profundidad que tiene esta ficha. Entre más grande sea este número
     * más cerca del jugador estará, es decir, estár encima de otras fichas.
     */
    public int nivelZ()
    {
        return nivelZ;
    }
    
    /** Cambia el nivel de la ficha al dado **/
    public void asignarNivelZ(int nivelZ)
    {
        this.nivelZ = nivelZ;
    }
    
    /**
     * Retorna true si esta ficha esta libre, es decir, no tiene fichas encima ni está "prensada"
     * por otras fichas adyacentes.
     */
    public boolean estaLibre()
    {
        java.util.List<Ficha> adjacentes = getIntersectingObjects(Ficha.class);
        for ( Ficha otra : adjacentes )
            if ( otra.nivelZ >= this.nivelZ )
                return false;
        
        return true;
    }
    
    /**
     * Resalta la ficha de tal forma que se distinga de las demás. Útil para indicar que la ficha
     * está seleccionada.
     */
    public void resaltar()
    {
        GreenfootImage actual = new GreenfootImage(getImage());
        actual.setColor(colorSeleccion);
        actual.fillRect( 2, 2, actual.getWidth() - 4, actual.getHeight() - 4 );
        setImage(actual);
    }
    
    /**
     * Quita el efecto de resaltado a la ficha
     */
    public void deseleccionar()
    {
        setImage(image);
    }
    
    /**
     * Act - do whatever the Ficha wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        if ( Greenfoot.mouseClicked(this) )
            resaltar();
    }
}
